# Copyright (c) 1999-2003 Xerox Corporation.  All Rights Reserved.
#
# Permission to use, copy, modify  and  distribute  without  charge
# this  software,  documentation, images, etc. is granted, provided
# that this copyright and the author"s name is retained.
#    
# A fee may be charged for this program ONLY to recover  costs  for
# distribution  (i.e.  media costs).  No profit can be made on this
# program.
#        
# The author assumes no responsibility for  disasters  (natural  or
# otherwise) as a consequence of use of this software.
#            
# Adam Stein (adam\@scan.mc.xerox.com)

use Time::ParseDate;

# For date/time parsing operation
local(%options) = ("WHOLE" => 1, "DATE_REQUIRED" => 1, "TIME_REQUIRED" => 1,
                   "NO_RELATIVE" => 1);

# Starting/Ending hours
my ($hour) = ["01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11",
	      "12", "No Time", "All Day"];

# Starting/Ending minutes
my ($minute) = ["00", "15", "30", "45"];

# Occurence choices
my ($occurs) = ["One Time", "Daily", "Weekly", "Every Two Weeks",
		"Monthly By Date", "Monthly By Weekday", "Yearly",
		"Monday Thru Friday", "Mon, Wed, Fri", "Tuesday, Thursday",
		"Repeat Every ..."
	       ];

# Privacy choices
my ($privacy) = ["Others See Time And Text", "Others See Time Only",
		 "Others See Nothing"
		];

# Units for reminders
my ($runits) = ["Mins", "Hrs", "Days"];

# Conversion factors for the various units
my (%convert) = ("Mins" => 60, "Hrs" => 60*60, "Days" => 24*60*60);

sub AddApptForm {
  my ($cgi) = shift,                            # CGI object
  my ($view) = shift,                           # Calendar view
  my ($date) = shift,				# The intial date for the form
  my (@user);					# User running this script

  &HTMLHeader($cgi, "f2", $cgi->param("key"));

  print $cgi->h1({align => "center"}, "Add Appointment\n");

  if ($mode & A_INSERT) {
    # Create form to add an appointment
    print $cgi->startform(-action => $cgi->url . "/f2"),
	  $cgi->hidden(-name => "action", -default => "addappt", -override =>1),
	  $cgi->hidden(-name => "key", -value => $cgi->param("key")),
	  $cgi->hidden(-name => "day", -value => $cgi->param("day")),
	  $cgi->hidden(-name => "month", -value => $cgi->param("month")),
	  $cgi->hidden(-name => "year", -value => $cgi->param("year")),
	  $cgi->hidden(-name => "view", -value => $cgi->param("view")),
	  $cgi->hidden(-name => "repeatval"),
	  $cgi->hidden(-name => "repeatunits"),

	  "&nbsp;&nbsp;Date:&nbsp;&nbsp;&nbsp;",
	  $cgi->textfield(-name => "apptdate",
			  -value => "$$date[1]/$$date[2]/$$date[0]"),
	  $cgi->br,

	  "&nbsp;&nbsp;Start:&nbsp;&nbsp;&nbsp;",
	  $cgi->textfield(-name => "start", -maxlength => 6, -size => 6,
			  -value => "08:00"),
	  $cgi->popup_menu(-name => "starthr", -values => $hour,
			   -default => "08", -onChange => "changeHour(0,this)"),
	  ":",
	  $cgi->popup_menu(-name => "startmin", -values => $minute,
			   -default => "00", -onChange => "changeMin(0, this)"),
	  $cgi->radio_group(-name => "startampm", -values => ["am", "pm"]),
	  $cgi->br,

	  "&nbsp;&nbsp;&nbsp;&nbsp;End:&nbsp;&nbsp;&nbsp;",
	  $cgi->textfield(-name => "end", -maxlength => 6, -size => 6,
			  -value => "09:00"),
	  $cgi->popup_menu(-name => "endhr", -values => $hour,
			   -default => "09", -onChange => "changeHour(1,this)"),
	  ":",
	  $cgi->popup_menu(-name => "endmin", -values => $minute,
			   -default => "00", -onChange => "changeMin(1, this)"),
	  $cgi->radio_group(-name => "endampm", -values => ["am", "pm"]),
	  $cgi->br,

	  $cgi->table(
	    $cgi->TR(
	      $cgi->td({-valign => "top"}, "What: "),
	      $cgi->td($cgi->textarea(-name => "what", -rows => 3, -cols =>21))
	    )
	  ),

	  $cgi->hr,

	  $cgi->table(
	    $cgi->TR(
	      $cgi->th({align => "left"}, "Frequency"),
	      $cgi->th({align => "left"}, "Reminders")
	    ),

	    $cgi->TR(
	      $cgi->td("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Occurs:&nbsp;",
		       $cgi->popup_menu(-name => "occurs", -values => $occurs,
		     		        -onChange => "repeatWin(this)")
	      ),

	      $cgi->td("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;",
		       $cgi->checkbox(-name =>"Beep", -value =>"Y",-checked=>1),
		       $cgi->textfield(-name => "btime", -value => 10,
				       -size => 4, -maxlength => 4),
		       $cgi->popup_menu(-name => "bunits", -values => $runits)
	      )
	    ),

	    $cgi->TR(
	      $cgi->td("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;",
		       "&nbsp;&nbsp;&nbsp;For:&nbsp;",
		       $cgi->textfield(-name => "numtimes", -value => "1",
				       -size => 8),
		       "occurrence(s)"
	      ),

	      $cgi->td("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;",
		       $cgi->checkbox(-name=>"Flash", -value =>"Y",-checked=>1),
		       $cgi->textfield(-name => "ftime", -value => 10,
				       -size => 4, -maxlength => 4),
		       $cgi->popup_menu(-name => "funits", -values => $runits)
	      )
	    ),

	    $cgi->TR(
	      $cgi->td("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Privacy:&nbsp;",
		       $cgi->popup_menu(-name => "privacy", -values => $privacy)
	      ),

	      $cgi->td("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;",
		       $cgi->checkbox(-name=>"Popup", -value =>"Y",-checked=>1),
		       $cgi->textfield(-name => "ptime", -value => 10,
				       -size => 4, -maxlength => 4),
		       $cgi->popup_menu(-name => "punits", -values => $runits)
	      )
	    ),

	    $cgi->TR(
	      $cgi->td(""
	      ),

	      $cgi->td("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;",
		       $cgi->checkbox(-name =>"Mail", -value =>"N",-checked=>0),
		       "&nbsp;&nbsp;",
		       $cgi->textfield(-name => "mtime", -value => "",
				       -size => 4, -maxlength => 4),
		       $cgi->popup_menu(-name => "munits", -values => $runits,
				        -default => "Hrs")
	      )
	    ),

	    $cgi->TR(
	      $cgi->td(""
	      ),

	      $cgi->td("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;",
		       "Mail To:&nbsp;",
		       $cgi->textfield(-name => "email",
				       -value=>$calendar{$cgi->param("key")}[1])
	      )
	    )
	  ),

	  $cgi->hr,

	  $cgi->center(
	    $cgi->p({-align => "center"}, $cgi->submit(-name => "Insert"),
		    $cgi->defaults("Clear"))
	  ),

	  $cgi->endform;
  } else {
    @user = getpwuid($>);

    print p("Sorry.  It appears that the account running ", strong(em("wcal")),
            "($user[0]) does not have the authority to add appointments ",
            "to this calendar."),

          p("View the README file that came with ", strong(em("wcal")),
            "(or appropriate man pages) to find out how to change ",
            "permissions to allow adding appointments to a CDE calendar.");
  }
}

sub AddAppt {
  my ($cgi) = shift,                            # CGI object
  my (%appt),					# Appointment structure
  my ($caladdr),                                # Calendar addr to connect to
  my ($errmsg),					# Error msg from adding appt
  my ($etime),                                  # Ending time
  my ($js1),                                    # JavaScript to update user
  my ($js2),                                    # JavaScript to close update win
  my ($key),					# Key to calendar to display
  my ($numtimes),				# # of times an appt will be
  my ($session),                                # Handle to CDE calendar
  my ($status) = 0,				# Flag to add appt or not
  my ($stime);                                  # Starting time

  # Make sure the fields we need contain values
  if ($cgi->param("apptdate") eq "") {
    print "<SCRIPT>\n",
	  "alert(\"WARNING: Invalid date\\n\\n",
	  "Appointment will NOT be added\");",
	  "</SCRIPT>\n";

    return;
  }

  if ($cgi->param("start") eq "") {
    print "<SCRIPT>\n",
	  "alert(\"WARNING: Invalid starting time\\n\\n",
	  "Appointment will NOT be added\");",
	  "</SCRIPT>\n";

    return;
  }

  if ($cgi->param("end") eq "") {
    print "<SCRIPT>\n",
	  "alert(\"WARNING: Invalid ending time\\n\\n",
	  "Appointment will NOT be added\");",
	  "</SCRIPT>\n";

    return;
  }

  # See if we were given a full calendar address or
  # if we need to look it up
  $key = $cgi->param("key");
  $caladdr = (($key =~ /@/) ? $key : $calendar{$key}[1]);

  # Open connection to calendar
  if (($caladdr eq "") || (&OpenCalendar($caladdr, \$session) ne "")) {
    # Can't connect with CSA
    print "<SCRIPT>\n",
	  "alert(\"WARNING: Can't connect to [$key]\\n\\n",
	  "Appointment will NOT be added\");",
	  "</SCRIPT>\n";

    $USECSA = -1;

    return;
  }

  # Set up entry to add
  $appt{"Type"} = &data("UINT32", 0);
  $appt{"Subtype"} = &data("STRING", "Subtype Appointment");

  $stime = &FormatDate($cgi->param("apptdate"), $cgi->param("start"), undef,
		       $cgi->param("startampm"), undef);

  $etime = &FormatDate($cgi->param("apptdate"), $cgi->param("start"), 
		       $cgi->param("end"), $cgi->param("startampm"),
		       $cgi->param("endampm"));

  $appt{"Start Date"} = &data("DATE TIME", parsedate($stime, %options));
  $appt{"End Date"} = &data("DATE TIME", parsedate($etime, %options));
  $appt{"Show Time"} = &data("SINT32", 1);
  $appt{"Summary"} = &data("STRING", $cgi->param("what"));

  # Set up reminders
  $status = &SetupReminder("Beep", $cgi->param("btime"), $cgi->param("bunits"),
			   "Audio Reminder", undef, \%appt)
	if (defined($cgi->param("Beep")));

  $status = &SetupReminder("Flash", $cgi->param("ftime"),
			   $cgi->param("funits"), "Flashing Reminder", undef,
			   \%appt)
	if (defined($cgi->param("Flash")));

  $status = &SetupReminder("Popup", $cgi->param("ptime"),
			   $cgi->param("punits"), "Popup Reminder", undef,
			   \%appt)
	if (defined($cgi->param("Popup")));

  $status = &SetupReminder("Mail", $cgi->param("mtime"),
			   $cgi->param("munits"), "Mail Reminder",
			   $cgi->param("email"), \%appt)
	if (defined($cgi->param("Mail")));

  if (!$status) {
    # Set up frequency
    if ($cgi->param("occurs") eq "Daily") {
      $appt{"Recurrence Rule"} = &data("STRING",
				       "D1 #" . $cgi->param("numtimes"));
    } elsif ($cgi->param("occurs") eq "Weekly") {
      $appt{"Recurrence Rule"} = &data("STRING",
				       "W1 #" . $cgi->param("numtimes"));
    } elsif ($cgi->param("occurs") eq "Every Two Weeks") {
      $appt{"Recurrence Rule"} = &data("STRING",
				       "W2 #" . $cgi->param("numtimes"));
    } elsif ($cgi->param("occurs") eq "Monthly By Date") {
      $appt{"Recurrence Rule"} = &data("STRING",
				       "MD1 #" . $cgi->param("numtimes"));
    } elsif ($cgi->param("occurs") eq "Monthly By Weekday") {
      $appt{"Recurrence Rule"} = &data("STRING",
				       "MP1 #" . $cgi->param("numtimes"));
    } elsif ($cgi->param("occurs") eq "Yearly") {
      $appt{"Recurrence Rule"} = &data("STRING",
				       "YM1 #" . $cgi->param("numtimes"));
    } elsif ($cgi->param("occurs") eq "Monday Thru Friday") {
      $appt{"Recurrence Rule"} = &data("STRING",
				       "W1 MO TU WE TH FR #" .
				       $cgi->param("numtimes"));
    } elsif ($cgi->param("occurs") eq "Mon, Wed, Fri") {
      $appt{"Recurrence Rule"} = &data("STRING",
				       "W1 MO WE FR #".$cgi->param("numtimes"));
    } elsif ($cgi->param("occurs") eq "Tuesday, Thursday") {
      $appt{"Recurrence Rule"} = &data("STRING",
				       "W1 TU TH #" . $cgi->param("numtimes"));
    } elsif ($cgi->param("occurs") eq "Repeat Every ...") {
      # Calculate the number of appointments that actually will be
      $numtimes = int($cgi->param("numtimes")/$cgi->param("repeatval")) +
		  (($cgi->param("numtimes")%$cgi->param("repeatval")) ? 1 : 0);

      if ($cgi->param("repeatunits") == 0) {
        # Every X days
        $appt{"Recurrence Rule"} = &data("STRING",
				         "D" . $cgi->param("repeatval") . " #" .
				         $numtimes);
      } elsif ($cgi->param("repeatunits") == 1) {
        # Every X weeks
        $appt{"Recurrence Rule"} = &data("STRING",
				         "W" . $cgi->param("repeatval") . " #" .
				         $numtimes);
      } else {
        # Every X months
        $appt{"Recurrence Rule"} = &data("STRING",
				         "MD" . $cgi->param("repeatval") ." #" .
				         $numtimes);
      }
    }

    # Set up Privacy
    if ($cgi->param("privacy") eq "Others See Time And Text") {
      $appt{"Classification"} = &data("UINT32", 0);
    } elsif ($cgi->param("privacy") eq "Others See Time Only") {
      $appt{"Classification"} = &data("UINT32", 2);
    } elsif ($cgi->param("privacy") eq "Others See Nothing") {
      $appt{"Classification"} = &data("UINT32", 1);
    }

    eval { $session->add_entry(%appt); };
    $errmsg = $@;
  }

  # Close connection to calendar
  &CloseCalendar($session);

  if ($errmsg) {
    print "<SCRIPT>\n",
	  "alert(\"WARNING: $errmsg\\n\\n",
	  "Appointment may not have been added\");",
	  "</SCRIPT>\n";
  } elsif ($status) {
    print "<SCRIPT>\n",
	  "alert(\"WARNING: there were errors\\n\\n",
	  "Appointment was NOT added!\");",
	  "</SCRIPT>\n";
  }

  # Go back to previous calendar view
  print "<SCRIPT>history.go(-2)</SCRIPT>\n";
}

# Set up a particular reminder
sub SetupReminder {
  my ($name) = shift,				# Name of reminder
  my ($rtime) = shift,				# Time before event to remind
  my ($runits) = shift,				# Units of $rtime
  my ($rindx) = shift,				# Reminder index into hash
  my ($email) = shift,				# Email addr for mail reminder
  my ($appt) = shift,				# Appointment structure
  my ($data),					# Reminder data
  my ($status) = 0;				# Validation results

  # Set up for a reminder if necessary
  if ($rtime ne "") {
    if ($name eq "Mail") {
      if ($email ne "") {
	$data = $email;
      } else {
        print "<SCRIPT>\n",
	      "alert(\"Email address is missing, not adding appointment\");",
	      "</SCRIPT>\n";

        $status = 1;
      }
    }
  } else {
    print "<SCRIPT>\n",
	  "alert(\"$name Repeat Count not set, not adding appointment\");",
	  "</SCRIPT>\n";

    $status = 1;
  }

  if (!$status) {
    $$appt{$rindx} = &data("REMINDER", {
			                 "repeat_count" => "0",
			                 "snooze_time" => "",
			                 "lead_time" => $rtime *
							$convert{$runits},
		                         "data" => $data
		                       });
  }

  return($status);
}

# Convert date/time to a 24-hour time with a 4-digit year
sub FormatDate {
  my ($datestr) = shift,			# Date string
  my ($stimestr) = shift,			# Starting time string
  my ($etimestr) = shift,			# Ending time string
  my ($sampm) = shift,				# Starting AM/PM setting
  my ($eampm) = shift,				# Ending AM/PM setting
  my (@date),					# Split apart date
  my (@etime),					# Split apart ending time
  my (@next),					# Next day'sdate
  my (@stime),					# Split apart starting time
  my (@time);					# Modified time

  # Turn year into a 4-digit value if necessary
  @date = split("/", $datestr);
  $date[2] += ($date[2] < 70) ? 2000 : 1900 if (length($date[2]) == 2);

  # Turn starting time into a 24-hour format
  @stime = split(":", $stimestr);
  $stime[0] += 12 if ($sampm eq "pm");

  if (defined($etimestr)) {
    # Turn ending time into a 24-hour format
    @etime = split(":", $etimestr);
    $etime[0] += 12 if ($eampm eq "pm");

    # Figure out if we need to change the ending date to the next day.
    # This will occur if:
    #   1) starting & ending AM/PM setting are the same and
    #      the ending hour is less than the starting hour
    #   2) starting AM/PM setting is PM and ending AM/PM setting is AM
    if ((($sampm eq $eampm) && ($etime[0] < $stime[0])) ||
	(($sampm eq "pm") && ($eampm eq "am"))) {
      @next = Add_Delta_Days(($date[2], $date[0], $date[1]), 1);

      $date[0] = $next[1];
      $date[1] = $next[2];
      $date[2] = $next[0];
    }

    @time = @etime;
  } else {
    @time = @stime;
  }

  return(join("/", @date) . " " . join(":", @time) . ":00");
}

1;

